@@ -8,14 +8,14 @@ proto = txtorcon.TorControlProtocol()
 state = txtorcon.TorState(proto)
 
 data = open('consensus', 'r').read()
-routers = 4724  # number of routers in above file
-iters = 100
+routers = 5494  # number of routers in above file
+iters = 5
 
 start = time()
 if False:
     cProfile.run('state._update_network_status(data)')
 else:
-    for x in xrange(iters):
+    for x in range(iters):
         state._update_network_status(data)
 diff = time() - start
-print "%fs: %f microdescriptors/second" % (diff, (routers * iters) / diff)
+print("%fs: %f microdescriptors/second" % (diff, (routers * iters) / diff))
