@@ -5,6 +5,7 @@ import sys
 from mock import patch
 from mock import Mock, MagicMock
 from unittest import skipIf
+from binascii import b2a_base64
 
 from zope.interface import implementer, directlyProvides
 
@@ -547,6 +548,18 @@ class EndpointTests(unittest.TestCase):
         ep._tor_progress_update(40, "FOO", "foo to bar")
         return ep
 
+    def test_single_hop_non_ephemeral(self, ftb):
+        control_ep = Mock()
+        control_ep.connect = Mock(return_value=defer.succeed(None))
+        directlyProvides(control_ep, IStreamClientEndpoint)
+        with self.assertRaises(ValueError) as ctx:
+            TCPHiddenServiceEndpoint.system_tor(
+                self.reactor, control_ep, 1234,
+                ephemeral=False,
+                single_hop=True,
+            )
+        self.assertIn("single_hop=", str(ctx.exception))
+
     def test_progress_updates_global_tor(self, ftb):
         with patch('txtorcon.endpoints.get_global_tor_instance') as tor:
             ep = TCPHiddenServiceEndpoint.global_tor(self.reactor, 1234)
@@ -704,6 +717,172 @@ class EndpointTests(unittest.TestCase):
         self.assertEqual(ep.local_port, 1234)
         self.assertEqual(ep.hidden_service_dir, '/foo/bar')
 
+    def test_parse_via_plugin_key_from_file(self, ftb):
+        tmp = self.mktemp()
+        os.mkdir(tmp)
+        with open(os.path.join(tmp, 'some_data'), 'wb') as f:
+            f.write(b'ED25519-V3:deadbeefdeadbeef\n')
+
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
+        config = TorConfig()
+        config.post_bootstrap = defer.succeed(config)
+        from txtorcon import torconfig
+        torconfig._global_tor_config = None
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+        )
+        ep = serverFromString(
+            self.reactor,
+            'onion:88:localPort=1234:privateKeyFile={}'.format(os.path.join(tmp, 'some_data')),
+        )
+        self.assertEqual(ep.public_port, 88)
+        self.assertEqual(ep.local_port, 1234)
+        self.assertEqual(ep.private_key, "ED25519-V3:deadbeefdeadbeef")
+
+    def test_parse_via_plugin_key_from_v3_private_file(self, ftb):
+        tmp = self.mktemp()
+        os.mkdir(tmp)
+        with open(os.path.join(tmp, 'some_data'), 'wb') as f:
+            f.write(b'== ed25519v1-secret: type0 ==\x00\x00\x00H\x9e\xa6j\x0e\x98\x85\xa9\xec\xee@\x9d&\xe2\xbfe\xc9\x90\xb9\xcb\xb2g\xb0\xab\xe4\xd0\x14c\xb0\xb2\x9dX\xfa\xaa\xf8,di8\xec\xc6\x82t\xd0A\x16>u\xde\xc6&\x82\x03\x1app\x18c`T\xc3\xdc\x1a\xca')
+
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
+        config = TorConfig()
+        config.post_bootstrap = defer.succeed(config)
+        from txtorcon import torconfig
+        torconfig._global_tor_config = None
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+        )
+        ep = serverFromString(
+            self.reactor,
+            'onion:88:localPort=1234:privateKeyFile={}'.format(os.path.join(tmp, 'some_data')),
+        )
+        self.assertEqual(ep.public_port, 88)
+        self.assertEqual(ep.local_port, 1234)
+        self.assertTrue("\n" not in ep.private_key)
+        self.assertEqual(
+            ep.private_key,
+            u"ED25519-V3:" + b2a_base64(b"H\x9e\xa6j\x0e\x98\x85\xa9\xec\xee@\x9d&\xe2\xbfe\xc9\x90\xb9\xcb\xb2g\xb0\xab\xe4\xd0\x14c\xb0\xb2\x9dX\xfa\xaa\xf8,di8\xec\xc6\x82t\xd0A\x16>u\xde\xc6&\x82\x03\x1app\x18c`T\xc3\xdc\x1a\xca").decode('ascii').strip(),
+        )
+
+    def test_parse_via_plugin_key_from_v2_private_file(self, ftb):
+        tmp = self.mktemp()
+        os.mkdir(tmp)
+        with open(os.path.join(tmp, 'some_data'), 'w') as f:
+            f.write('-----BEGIN RSA PRIVATE KEY-----\nthekeyblob\n-----END RSA PRIVATE KEY-----\n')
+
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
+        config = TorConfig()
+        config.post_bootstrap = defer.succeed(config)
+        from txtorcon import torconfig
+        torconfig._global_tor_config = None
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+        )
+        ep = serverFromString(
+            self.reactor,
+            'onion:88:localPort=1234:privateKeyFile={}'.format(os.path.join(tmp, 'some_data')),
+        )
+        self.assertEqual(ep.public_port, 88)
+        self.assertEqual(ep.local_port, 1234)
+        self.assertEqual(
+            ep.private_key,
+            u"RSA1024:thekeyblob",
+        )
+
+    def test_parse_via_plugin_key_from_invalid_private_file(self, ftb):
+        tmp = self.mktemp()
+        os.mkdir(tmp)
+        with open(os.path.join(tmp, 'some_data'), 'w') as f:
+            f.write('nothing to see here\n')
+
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
+        config = TorConfig()
+        config.post_bootstrap = defer.succeed(config)
+        from txtorcon import torconfig
+        torconfig._global_tor_config = None
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+        )
+
+        with self.assertRaises(ValueError):
+            serverFromString(
+                self.reactor,
+                'onion:88:localPort=1234:privateKeyFile={}'.format(os.path.join(tmp, 'some_data')),
+            )
+
+    def test_parse_via_plugin_single_hop(self, ftb):
+        tmp = self.mktemp()
+        os.mkdir(tmp)
+        with open(os.path.join(tmp, 'some_data'), 'wb') as f:
+            f.write(b'ED25519-V3:deadbeefdeadbeef\n')
+
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
+        config = TorConfig()
+        config.post_bootstrap = defer.succeed(config)
+        from txtorcon import torconfig
+        torconfig._global_tor_config = None
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+        )
+        ep = serverFromString(
+            self.reactor,
+            'onion:88:localPort=1234:singleHop=True:privateKeyFile={}'.format(os.path.join(tmp, 'some_data')),
+        )
+        self.assertEqual(ep.public_port, 88)
+        self.assertEqual(ep.local_port, 1234)
+        self.assertEqual(ep.private_key, "ED25519-V3:deadbeefdeadbeef")
+        self.assertTrue(ep.single_hop)
+
+    def test_parse_via_plugin_single_hop_explicit_false(self, ftb):
+        tmp = self.mktemp()
+        os.mkdir(tmp)
+        with open(os.path.join(tmp, 'some_data'), 'wb') as f:
+            f.write(b'ED25519-V3:deadbeefdeadbeef\n')
+
+        # make sure we have a valid thing from get_global_tor without
+        # actually launching tor
+        config = TorConfig()
+        config.post_bootstrap = defer.succeed(config)
+        from txtorcon import torconfig
+        torconfig._global_tor_config = None
+        get_global_tor(
+            self.reactor,
+            _tor_launcher=lambda react, config, progress_updates=None: defer.succeed(config)
+        )
+        ep = serverFromString(
+            self.reactor,
+            'onion:88:localPort=1234:singleHop=false:privateKeyFile={}'.format(os.path.join(tmp, 'some_data')),
+        )
+        self.assertEqual(ep.public_port, 88)
+        self.assertEqual(ep.local_port, 1234)
+        self.assertEqual(ep.private_key, "ED25519-V3:deadbeefdeadbeef")
+        self.assertFalse(ep.single_hop)
+
+    def test_parse_via_plugin_single_hop_bogus(self, ftb):
+        with self.assertRaises(ValueError):
+            serverFromString(
+                self.reactor,
+                'onion:88:singleHop=yes_please',
+            )
+
+    def test_parse_via_plugin_key_and_keyfile(self, ftb):
+        with self.assertRaises(ValueError):
+            serverFromString(
+                self.reactor,
+                'onion:88:privateKeyFile=foo:privateKey=blarg'
+            )
+
     def test_parse_via_plugin_key_and_dir(self, ftb):
         with self.assertRaises(ValueError):
             serverFromString(
@@ -1681,6 +1860,26 @@ class TestSocksFactory(unittest.TestCase):
         self.assertTrue(isinstance(ep, UNIXClientEndpoint))
         self.assertEqual("/tmp/boom", ep._path)
 
+    @defer.inlineCallbacks
+    def test_unix_socket_bad(self):
+        reactor = Mock()
+        cp = Mock()
+        cp.get_conf = Mock(
+            return_value=defer.succeed({
+                'SocksPort': ['unix:bad worse wosrt']
+            })
+        )
+        the_error = Exception("a bad thing")
+
+        def boom(*args, **kw):
+            raise the_error
+
+        with patch('txtorcon.endpoints.available_tcp_port', lambda r: 1234):
+            with patch('txtorcon.torconfig.UNIXClientEndpoint', boom):
+                yield _create_socks_endpoint(reactor, cp)
+        errs = self.flushLoggedErrors()
+        self.assertEqual(errs[0].value, the_error)
+
     @defer.inlineCallbacks
     def test_nothing_exists(self):
         reactor = Mock()
