@@ -5,9 +5,12 @@ from twisted.trial import unittest
 from twisted.internet import defer
 from twisted.internet.address import IPv4Address
 from twisted.internet.protocol import Protocol
+from twisted.internet.interfaces import IStreamClientEndpoint
 from twisted.test import proto_helpers
 from twisted.test.iosim import connect, FakeTransport
 
+from zope.interface import directlyProvides
+
 from txtorcon import socks
 
 
@@ -511,6 +514,7 @@ class SocksConnectTests(unittest.TestCase):
     @defer.inlineCallbacks
     def test_connect_deferred_proxy(self):
         socks_ep = Mock()
+        directlyProvides(socks_ep, IStreamClientEndpoint)
         transport = proto_helpers.StringTransport()
 
         def connect(factory):
@@ -533,6 +537,25 @@ class SocksConnectTests(unittest.TestCase):
         proto = yield ep.connect(factory)
         self.assertEqual(proto, protocol)
 
+    @defer.inlineCallbacks
+    def test_connect_deferred_proxy_wrong_return(self):
+
+        class NotAnEndpoint(object):
+            "definitely doesn't implement IStreamClientEndpoint"
+        not_an_endpoint = NotAnEndpoint()
+        factory = Mock()
+        ep = socks.TorSocksEndpoint(
+            socks_endpoint=defer.succeed(not_an_endpoint),
+            host=u'meejah.ca',
+            port=443,
+        )
+        with self.assertRaises(ValueError) as ctx:
+            yield ep.connect(factory)
+        self.assertIn(
+            "must resolve to an IStreamClientEndpoint provider",
+            str(ctx.exception),
+        )
+
     @defer.inlineCallbacks
     def test_connect_tls(self):
         socks_ep = Mock()
@@ -554,6 +577,34 @@ class SocksConnectTests(unittest.TestCase):
         proto = yield ep.connect(factory)
         self.assertEqual(proto, protocol)
 
+    @defer.inlineCallbacks
+    def test_connect_tls_context(self):
+        socks_ep = Mock()
+        transport = proto_helpers.StringTransport()
+
+        def connect(factory):
+            factory.startFactory()
+            proto = factory.buildProtocol("addr")
+            proto.makeConnection(transport)
+            self.assertEqual(b'\x05\x01\x00', transport.value())
+            proto.dataReceived(b'\x05\x00')
+            proto.dataReceived(b'\x05\x00\x00\x01\x00\x00\x00\x00\x00\x00')
+            return proto
+        socks_ep.connect = connect
+        protocol = Mock()
+        factory = Mock()
+        factory.buildProtocol = Mock(return_value=protocol)
+
+        from OpenSSL import SSL
+
+        class CertificateOptions(object):
+            def getContext(self, *args, **kw):
+                return SSL.Context(SSL.TLSv1_METHOD)
+
+        ep = socks.TorSocksEndpoint(socks_ep, u'meejah.ca', 443, tls=CertificateOptions())
+        proto = yield ep.connect(factory)
+        self.assertEqual(proto, protocol)
+
     @defer.inlineCallbacks
     def test_connect_socks_error(self):
         socks_ep = Mock()
